@@ -126,6 +126,11 @@ class VcTreeStore(tree.DiffTreeStore):
     def __init__(self):
         tree.DiffTreeStore.__init__(self, 1, [str] * 5)
 
+    def get_file_path(self, it):
+        # Use instead of value_path; does not incorrectly decode
+        return self.get_value(it, self.column_index(tree.COL_PATH, 0))
+
+
 ################################################################################
 # filters
 ################################################################################
@@ -177,6 +182,13 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         "ignored": ("VcShowIgnored", entry_ignored),
     }
 
+    file_encoding = sys.getfilesystemencoding()
+
+    @classmethod
+    def display_path(cls, bytes):
+        encodings = (cls.file_encoding,)
+        return misc.fallback_decode(bytes, encodings, lossy=True)
+
     def __init__(self):
         melddoc.MeldDoc.__init__(self)
         gnomeglade.Component.__init__(self, "vcview.ui", "vcview",
@@ -408,18 +420,16 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         return recent.TYPE_VC, [self.location]
 
     def recompute_label(self):
-        location = self.location
-        if isinstance(location, str):
-            location = location.decode(
-                sys.getfilesystemencoding(), 'replace')
+        location = self.display_path(self.location)
         self.label_text = os.path.basename(location)
         # TRANSLATORS: This is the location of the directory the user is diffing
         self.tooltip_text = _("%s: %s") % (_("Location"), location)
         self.label_changed()
 
     def _search_recursively_iter(self, iterstart):
-        rootname = self.model.value_path(iterstart, 0)
-        prefixlen = len(self.location) + 1
+        rootname = self.model.get_file_path(iterstart)
+        prefixlen = len(rootname) + 1
+        display_prefix = len(self.display_path(rootname)) + 1
         symlinks_followed = set()
         todo = [(self.model.get_path(iterstart), rootname)]
 
@@ -428,7 +438,6 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         filters = [a[1] for a in self.state_actions.values() if
                    active_action(a[0]) and a[1]]
 
-        yield _("Scanning %s") % rootname
         self.vc.cache_inventory(rootname)
         while todo:
             # This needs to happen sorted and depth-first in order for our row
@@ -436,7 +445,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
             todo.sort()
             treepath, path = todo.pop(0)
             it = self.model.get_iter(treepath)
-            yield _("Scanning %s") % path[prefixlen:]
+            yield _("Scanning %s") % self.display_path(path)[display_prefix:]
 
             entries = self.vc.listdir(path)
             entries = [e for e in entries if any(f(e) for f in filters)]
@@ -446,7 +455,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
                         st = os.lstat(e.path)
                     # Covers certain unreadable symlink cases; see bgo#585895
                     except OSError as err:
-                        error_string = "%s: %s" % (e.path, err.strerror)
+                        error_string = "%r: %s" % (e.path, err.strerror)
                         self.model.add_error(it, error_string, 0)
                         continue
 
@@ -499,7 +508,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
             else:
                 self.treeview.expand_row(path, False)
         else:
-            path = self.model.value_path(it, 0)
+            path = self.model.get_file_path(it)
             if not self.model.is_folder(it, 0, path):
                 self.run_diff(path)
 
@@ -509,7 +518,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
             return
 
         left_is_local = self.props.left_is_local
-        basename = os.path.basename(path)
+        basename = self.display_path(os.path.basename(path))
         meta = {
             'parent': self,
             'prompt_resolve': False,
@@ -613,7 +622,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
             selection = self.treeview.get_selection()
         model, rows = selection.get_selected_rows()
         if hasattr(self.vc, 'update_actions_for_paths'):
-            paths = [self.model.value_path(model.get_iter(r), 0) for r in rows]
+            paths = [self.model.get_file_path(model.get_iter(r)) for r in rows]
             states = [self.model.get_state(model.get_iter(r), 0) for r in rows]
             action_sensitivity = {
                 "VcCompare": False,
@@ -637,7 +646,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
 
     def _get_selected_files(self):
         model, rows = self.treeview.get_selection().get_selected_rows()
-        sel = [self.model.value_path(self.model.get_iter(r), 0) for r in rows]
+        sel = [self.model.get_file_path(self.model.get_iter(r)) for r in rows]
         # Remove empty entries and trailing slashes
         return [x[-1] != "/" and x or x[:-1] for x in sel if x is not None]
 
@@ -793,7 +802,7 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
         self._open_files(self._get_selected_files())
 
     def refresh(self):
-        self.set_location(self.model.value_path(self.model.get_iter_first(), 0))
+        self.set_location(self.model.get_file_path(self.model.get_iter_first()))
 
     def refresh_partial(self, where):
         if not self.actiongroup.get_action("VcFlatten").get_active():
@@ -827,25 +836,26 @@ class VcView(melddoc.MeldDoc, gnomeglade.Component):
     def on_file_changed(self, filename):
         it = self.find_iter_by_name(filename)
         if it:
-            path = self.model.value_path(it, 0)
+            path = self.model.get_file_path(it)
             self.vc.update_file_state(path)
             files = self.vc.lookup_files([], [(os.path.basename(path), path)])[1]
             for e in files:
                 if e.path == path:
-                    prefixlen = 1 + len( self.model.value_path( self.model.get_iter_first(), 0 ) )
+                    prefixlen = 1 + len(
+                        self.model.get_file_path(self.model.get_iter_first()))
                     self._update_item_state( it, e, e.parent[prefixlen:])
                     return
 
     def find_iter_by_name(self, name):
         it = self.model.get_iter_first()
-        path = self.model.value_path(it, 0)
+        path = self.model.get_file_path(it)
         while it:
             if name == path:
                 return it
             elif name.startswith(path):
                 child = self.model.iter_children( it )
                 while child:
-                    path = self.model.value_path(child, 0)
+                    path = self.model.get_file_path(child)
                     if name == path:
                         return child
                     elif name.startswith(path):
